@@ -15,7 +15,8 @@
 static void
 gs_plugins_dpkg_func (GsPluginLoader *plugin_loader)
 {
-	g_autoptr(GsApp) app = NULL;
+	GsAppList *list;
+	GsApp *app;
 	g_autoptr(GsPluginJob) plugin_job = NULL;
 	g_autoptr(GError) error = NULL;
 	g_autofree gchar *fn = NULL;
@@ -31,12 +32,16 @@ gs_plugins_dpkg_func (GsPluginLoader *plugin_loader)
 	fn = gs_test_get_filename (TESTDATADIR, "chiron-1.1-1.deb");
 	g_assert (fn != NULL);
 	file = g_file_new_for_path (fn);
-	plugin_job = gs_plugin_job_file_to_app_new (file, GS_PLUGIN_FILE_TO_APP_FLAGS_NONE);
-	app = gs_plugin_loader_job_process_app (plugin_loader, plugin_job, NULL, &error);
+	plugin_job = gs_plugin_job_file_to_app_new (file, GS_PLUGIN_FILE_TO_APP_FLAGS_NONE,
+						    GS_PLUGIN_REFINE_REQUIRE_FLAGS_NONE);
+	gs_plugin_loader_job_process (plugin_loader, plugin_job, NULL, &error);
+	list = gs_plugin_job_file_to_app_get_result_list (GS_PLUGIN_JOB_FILE_TO_APP (plugin_job));
 	gs_test_flush_main_context ();
 	g_assert_no_error (error);
-	g_assert (app != NULL);
-	g_assert_cmpstr (gs_app_get_source_default (app), ==, "chiron");
+	g_assert_nonnull (list);
+	g_assert_cmpuint (gs_app_list_length (list), ==, 1);
+	app = gs_app_list_index (list, 0);
+	g_assert_cmpstr (gs_app_get_default_source (app), ==, "chiron");
 	g_assert_cmpstr (gs_app_get_url (app, AS_URL_KIND_HOMEPAGE), ==, "http://127.0.0.1/");
 	g_assert_cmpstr (gs_app_get_name (app), ==, "chiron");
 	g_assert_cmpstr (gs_app_get_version (app), ==, "1.1-1");
