@@ -4,7 +4,7 @@
  * Copyright (C) 2008 Richard Hughes <richard@hughsie.com>
  * Copyright (C) 2015 Kalev Lember <klember@redhat.com>
  *
- * SPDX-License-Identifier: GPL-2.0+
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "config.h"
@@ -24,7 +24,7 @@
  * - No ordered list support
  * - No blockquote section support
  * - No image support
- * - No links or email support
+ * - No email support
  * - No backslash escapes support
  * - No HTML escaping support
  * - Auto-escapes certain word patterns, like http://
@@ -42,6 +42,10 @@ typedef enum {
 	GS_MARKDOWN_MODE_PARA,
 	GS_MARKDOWN_MODE_H1,
 	GS_MARKDOWN_MODE_H2,
+	GS_MARKDOWN_MODE_H3,
+	GS_MARKDOWN_MODE_H4,
+	GS_MARKDOWN_MODE_H5,
+	GS_MARKDOWN_MODE_H6,
 	GS_MARKDOWN_MODE_UNKNOWN
 } GsMarkdownMode;
 
@@ -56,9 +60,20 @@ typedef struct {
 	const gchar *h1_end;
 	const gchar *h2_start;
 	const gchar *h2_end;
+	const gchar *h3_start;
+	const gchar *h3_end;
+	const gchar *h4_start;
+	const gchar *h4_end;
+	const gchar *h5_start;
+	const gchar *h5_end;
+	const gchar *h6_start;
+	const gchar *h6_end;
 	const gchar *bullet_start;
 	const gchar *bullet_end;
 	const gchar *rule;
+	const gchar *link_prefix;
+	const gchar *link_mid;
+	const gchar *link_suffix;
 } GsMarkdownTags;
 
 struct _GsMarkdown {
@@ -115,26 +130,79 @@ gs_markdown_to_text_line_is_rule (const gchar *line)
 }
 
 static gboolean
-gs_markdown_to_text_line_is_bullet (const gchar *line)
+gs_markdown_to_text_line_is_bullet (const gchar **pline)
 {
-	return (g_str_has_prefix (line, "- ") ||
-		g_str_has_prefix (line, "* ") ||
-		g_str_has_prefix (line, "+ ") ||
-		g_str_has_prefix (line, " - ") ||
-		g_str_has_prefix (line, " * ") ||
-		g_str_has_prefix (line, " + "));
+	const gchar *line = *pline;
+
+	/* skip leading spaces */
+	while (g_ascii_isspace (*line))
+		line++;
+
+	if (g_str_has_prefix (line, "- ") ||
+	    g_str_has_prefix (line, "* ") ||
+	    g_str_has_prefix (line, "+ ")) {
+		*pline = line + 2;
+		return TRUE;
+	}
+
+	return FALSE;
 }
 
 static gboolean
-gs_markdown_to_text_line_is_header1 (const gchar *line)
+gs_markdown_to_text_line_is_header_x (const gchar **pline,
+				      guint xx)
 {
-	return g_str_has_prefix (line, "# ");
+	const gchar *line = *pline;
+
+	while (*line == '#' && xx > 0) {
+		line++;
+		xx--;
+	}
+
+	if (xx == 0 && *line != '\0' && *line != '#') {
+		if (g_ascii_isspace (*line))
+			line++;
+		*pline = line;
+		return TRUE;
+	}
+
+	return FALSE;
 }
 
 static gboolean
-gs_markdown_to_text_line_is_header2 (const gchar *line)
+gs_markdown_to_text_line_is_header1 (const gchar **pline)
 {
-	return g_str_has_prefix (line, "## ");
+	return gs_markdown_to_text_line_is_header_x (pline, 1);
+}
+
+static gboolean
+gs_markdown_to_text_line_is_header2 (const gchar **pline)
+{
+	return gs_markdown_to_text_line_is_header_x (pline, 2);
+}
+
+static gboolean
+gs_markdown_to_text_line_is_header3 (const gchar **pline)
+{
+	return gs_markdown_to_text_line_is_header_x (pline, 3);
+}
+
+static gboolean
+gs_markdown_to_text_line_is_header4 (const gchar **pline)
+{
+	return gs_markdown_to_text_line_is_header_x (pline, 4);
+}
+
+static gboolean
+gs_markdown_to_text_line_is_header5 (const gchar **pline)
+{
+	return gs_markdown_to_text_line_is_header_x (pline, 5);
+}
+
+static gboolean
+gs_markdown_to_text_line_is_header6 (const gchar **pline)
+{
+	return gs_markdown_to_text_line_is_header_x (pline, 6);
 }
 
 static gboolean
@@ -512,6 +580,46 @@ gs_markdown_word_auto_format_urls (const gchar *text)
 	return g_strjoinv (" ", words);
 }
 
+static gchar *
+gs_markdown_replace_links (GsMarkdown *self,
+			   const gchar *text)
+{
+	GString *str = g_string_new ("");
+	const gchar *start, *mid, *end, *from;
+
+	/* it's: [title](https://....) */
+	from = text;
+	start = strchr (from, '[');
+	while (start != NULL) {
+		start += 1;
+		mid = strstr (start, "](");
+		if (mid != NULL) {
+			mid += 2;
+			end = strchr (mid, ')');
+			if (end != NULL) {
+				if (start > from)
+					g_string_append_len (str, from, start - from - 1);
+				g_string_append (str, self->tags.link_prefix);
+				g_string_append_len (str, mid, end - mid);
+				g_string_append (str, self->tags.link_mid);
+				g_string_append_len (str, start, mid - start - 2);
+				g_string_append (str, self->tags.link_suffix);
+				from = end + 1;
+				start = strchr (from, '[');
+			} else {
+				break;
+			}
+		} else {
+			break;
+		}
+	}
+
+	if (*from)
+		g_string_append (str, from);
+
+	return g_string_free (str, FALSE);
+}
+
 static void
 gs_markdown_flush_pending (GsMarkdown *self)
 {
@@ -560,6 +668,14 @@ gs_markdown_flush_pending (GsMarkdown *self)
 		copy = temp;
 	}
 
+	if (self->tags.link_prefix != NULL &&
+	    self->tags.link_mid != NULL &&
+	    self->tags.link_suffix != NULL) {
+		temp = gs_markdown_replace_links (self, copy);
+		g_free (copy);
+		copy = temp;
+	}
+
 	/* do formatting */
 	temp = gs_markdown_to_text_line_format (self, copy);
 	if (self->mode == GS_MARKDOWN_MODE_BULLETT) {
@@ -578,6 +694,26 @@ gs_markdown_flush_pending (GsMarkdown *self)
 					self->tags.h2_start,
 					temp,
 					self->tags.h2_end);
+	} else if (self->mode == GS_MARKDOWN_MODE_H3) {
+		g_string_append_printf (self->processed, "%s%s%s\n",
+					self->tags.h3_start,
+					temp,
+					self->tags.h3_end);
+	} else if (self->mode == GS_MARKDOWN_MODE_H4) {
+		g_string_append_printf (self->processed, "%s%s%s\n",
+					self->tags.h4_start,
+					temp,
+					self->tags.h4_end);
+	} else if (self->mode == GS_MARKDOWN_MODE_H5) {
+		g_string_append_printf (self->processed, "%s%s%s\n",
+					self->tags.h5_start,
+					temp,
+					self->tags.h5_end);
+	} else if (self->mode == GS_MARKDOWN_MODE_H6) {
+		g_string_append_printf (self->processed, "%s%s%s\n",
+					self->tags.h6_start,
+					temp,
+					self->tags.h6_end);
 	} else if (self->mode == GS_MARKDOWN_MODE_PARA ||
 		   self->mode == GS_MARKDOWN_MODE_RULE) {
 		g_string_append_printf (self->processed, "%s\n", temp);
@@ -630,29 +766,65 @@ gs_markdown_to_text_line_process (GsMarkdown *self, const gchar *line)
 	}
 
 	/* bullet */
-	ret = gs_markdown_to_text_line_is_bullet (line);
+	ret = gs_markdown_to_text_line_is_bullet (&line);
 	if (ret) {
 		gs_markdown_flush_pending (self);
 		self->mode = GS_MARKDOWN_MODE_BULLETT;
-		ret = gs_markdown_add_pending (self, &line[2]);
+		ret = gs_markdown_add_pending (self, line);
 		goto out;
 	}
 
 	/* header1 */
-	ret = gs_markdown_to_text_line_is_header1 (line);
+	ret = gs_markdown_to_text_line_is_header1 (&line);
 	if (ret) {
 		gs_markdown_flush_pending (self);
 		self->mode = GS_MARKDOWN_MODE_H1;
-		ret = gs_markdown_add_pending_header (self, &line[2]);
+		ret = gs_markdown_add_pending_header (self, line);
 		goto out;
 	}
 
 	/* header2 */
-	ret = gs_markdown_to_text_line_is_header2 (line);
+	ret = gs_markdown_to_text_line_is_header2 (&line);
 	if (ret) {
 		gs_markdown_flush_pending (self);
 		self->mode = GS_MARKDOWN_MODE_H2;
-		ret = gs_markdown_add_pending_header (self, &line[3]);
+		ret = gs_markdown_add_pending_header (self, line);
+		goto out;
+	}
+
+	/* header3 */
+	ret = gs_markdown_to_text_line_is_header3 (&line);
+	if (ret) {
+		gs_markdown_flush_pending (self);
+		self->mode = GS_MARKDOWN_MODE_H3;
+		ret = gs_markdown_add_pending_header (self, line);
+		goto out;
+	}
+
+	/* header4 */
+	ret = gs_markdown_to_text_line_is_header4 (&line);
+	if (ret) {
+		gs_markdown_flush_pending (self);
+		self->mode = GS_MARKDOWN_MODE_H4;
+		ret = gs_markdown_add_pending_header (self, line);
+		goto out;
+	}
+
+	/* header5 */
+	ret = gs_markdown_to_text_line_is_header5 (&line);
+	if (ret) {
+		gs_markdown_flush_pending (self);
+		self->mode = GS_MARKDOWN_MODE_H5;
+		ret = gs_markdown_add_pending_header (self, line);
+		goto out;
+	}
+
+	/* header6 */
+	ret = gs_markdown_to_text_line_is_header6 (&line);
+	if (ret) {
+		gs_markdown_flush_pending (self);
+		self->mode = GS_MARKDOWN_MODE_H6;
+		ret = gs_markdown_add_pending_header (self, line);
 		goto out;
 	}
 
@@ -687,13 +859,24 @@ gs_markdown_set_output_kind (GsMarkdown *self, GsMarkdownOutputKind output)
 		self->tags.strong_end = "</b>";
 		self->tags.code_start = "<tt>";
 		self->tags.code_end = "</tt>";
-		self->tags.h1_start = "<big>";
-		self->tags.h1_end = "</big>";
-		self->tags.h2_start = "<b>";
-		self->tags.h2_end = "</b>";
+		self->tags.h1_start = "\n<big>";
+		self->tags.h1_end = "</big>\n";
+		self->tags.h2_start = "\n<b>";
+		self->tags.h2_end = "</b>\n";
+		self->tags.h3_start = "\n<b>";
+		self->tags.h3_end = "</b>\n";
+		self->tags.h4_start = "\n<b>";
+		self->tags.h4_end = "</b>\n";
+		self->tags.h5_start = "\n<b>";
+		self->tags.h5_end = "</b>\n";
+		self->tags.h6_start = "\n<b>";
+		self->tags.h6_end = "</b>\n";
 		self->tags.bullet_start = "• ";
 		self->tags.bullet_end = "";
 		self->tags.rule = "⎯⎯⎯⎯⎯⎯⎯⎯⎯⎯⎯⎯⎯⎯⎯⎯⎯⎯⎯⎯⎯⎯\n";
+		self->tags.link_prefix = "<a href=\"";
+		self->tags.link_mid = "\">";
+		self->tags.link_suffix = "</a>";
 		self->escape = TRUE;
 		self->autolinkify = TRUE;
 		break;
@@ -709,9 +892,20 @@ gs_markdown_set_output_kind (GsMarkdown *self, GsMarkdownOutputKind output)
 		self->tags.h1_end = "</h1>";
 		self->tags.h2_start = "<h2>";
 		self->tags.h2_end = "</h2>";
+		self->tags.h3_start = "<h3>";
+		self->tags.h3_end = "</h3>";
+		self->tags.h4_start = "<h4>";
+		self->tags.h4_end = "</h4>";
+		self->tags.h5_start = "<h5>";
+		self->tags.h5_end = "</h5>";
+		self->tags.h6_start = "<h6>";
+		self->tags.h6_end = "</h6>";
 		self->tags.bullet_start = "<li>";
 		self->tags.bullet_end = "</li>";
 		self->tags.rule = "<hr>";
+		self->tags.link_prefix = "<a href=\"";
+		self->tags.link_mid = "\">";
+		self->tags.link_suffix = "</a>";
 		self->escape = TRUE;
 		self->autolinkify = TRUE;
 		break;
@@ -727,9 +921,20 @@ gs_markdown_set_output_kind (GsMarkdown *self, GsMarkdownOutputKind output)
 		self->tags.h1_end = "]";
 		self->tags.h2_start = "-";
 		self->tags.h2_end = "-";
+		self->tags.h3_start = "  ";
+		self->tags.h3_end = "  ";
+		self->tags.h4_start = "   ";
+		self->tags.h4_end = "   ";
+		self->tags.h5_start = "    ";
+		self->tags.h5_end = "    ";
+		self->tags.h6_start = "     ";
+		self->tags.h6_end = "     ";
 		self->tags.bullet_start = "* ";
 		self->tags.bullet_end = "";
 		self->tags.rule = " ----- \n";
+		self->tags.link_prefix = NULL;
+		self->tags.link_mid = NULL;
+		self->tags.link_suffix = NULL;
 		self->escape = FALSE;
 		self->autolinkify = FALSE;
 		break;
@@ -782,6 +987,7 @@ gs_markdown_parse (GsMarkdown *self, const gchar *markdown)
 	guint i;
 	guint len;
 	g_auto(GStrv) lines = NULL;
+	const gchar *output;
 
 	g_return_val_if_fail (GS_IS_MARKDOWN (self), NULL);
 
@@ -802,11 +1008,16 @@ gs_markdown_parse (GsMarkdown *self, const gchar *markdown)
 	gs_markdown_flush_pending (self);
 
 	/* remove trailing \n */
-	while (g_str_has_suffix (self->processed->str, "\n"))
+	while (self->processed->len > 0 && self->processed->str[self->processed->len - 1] == '\n')
 		g_string_set_size (self->processed, self->processed->len - 1);
 
+	/* skip leading \n, which can happen with headers in the Pango mode */
+	output = self->processed->str;
+	while (*output == '\n')
+		output++;
+
 	/* get a copy */
-	temp = g_strdup (self->processed->str);
+	temp = g_strdup (output);
 	g_string_truncate (self->pending, 0);
 	g_string_truncate (self->processed, 0);
 	return temp;
