@@ -29,6 +29,10 @@ populate_version_history (GsAppVersionHistoryDialog *dialog,
 			  GsApp			    *app)
 {
 	g_autoptr(GPtrArray) version_history = NULL;
+	const gchar *app_version;
+	GtkWidget *installed_row = NULL;
+	gboolean can_check_whether_the_version_is_installed;
+	gboolean have_installed;
 
 	/* remove previous */
 	gs_widget_remove_all (dialog->listbox, (GsRemoveFunc) gtk_list_box_remove);
@@ -40,26 +44,52 @@ populate_version_history (GsAppVersionHistoryDialog *dialog,
 		gs_app_version_history_row_set_always_expanded (GS_APP_VERSION_HISTORY_ROW (row), TRUE);
 		gs_app_version_history_row_set_info (GS_APP_VERSION_HISTORY_ROW (row),
 						     gs_app_get_version (app),
-						     gs_app_get_release_date (app), NULL);
+						     gs_app_get_release_date (app), NULL, FALSE);
 		gtk_list_box_append (GTK_LIST_BOX (dialog->listbox), row);
-		gtk_widget_set_visible (row, TRUE);
 		return;
 	}
 
+	app_version = gs_app_get_version_ui (app);
+	can_check_whether_the_version_is_installed = app_version != NULL && version_history->len > 1 && gs_app_is_installed (app);
+	have_installed = !can_check_whether_the_version_is_installed;
+
 	/* add each */
 	for (guint i = 0; i < version_history->len; i++) {
 		GtkWidget *row;
 		AsRelease *version = g_ptr_array_index (version_history, i);
+		gboolean is_installed = FALSE;
+
+		if (app_version != NULL) {
+			gint vercmp = as_vercmp (as_release_get_version (version), app_version, AS_VERCMP_FLAG_NONE);
+			if (can_check_whether_the_version_is_installed) {
+				is_installed = vercmp == 0;
+				have_installed = have_installed || is_installed;
+				/* in case the versions are in some odd order */
+				if (is_installed && installed_row != NULL)
+					gtk_widget_set_visible (installed_row, FALSE);
+			}
+			/* add the installed version just before the first lower version if not part of the list */
+			if (!have_installed && vercmp < 0) {
+				have_installed = TRUE;
+				row = gs_app_version_history_row_new ();
+				gs_app_version_history_row_set_always_expanded (GS_APP_VERSION_HISTORY_ROW (row), TRUE);
+				gs_app_version_history_row_set_info (GS_APP_VERSION_HISTORY_ROW (row),
+								     app_version,
+								     gs_app_get_release_date (app), NULL, TRUE);
+				gtk_list_box_append (GTK_LIST_BOX (dialog->listbox), row);
+				installed_row = row;
+			}
+		}
 
 		row = gs_app_version_history_row_new ();
 		gs_app_version_history_row_set_always_expanded (GS_APP_VERSION_HISTORY_ROW (row), TRUE);
 		gs_app_version_history_row_set_info (GS_APP_VERSION_HISTORY_ROW (row),
 						     as_release_get_version (version),
 						     as_release_get_timestamp (version),
-						     as_release_get_description (version));
+						     as_release_get_description (version),
+						     is_installed);
 
 		gtk_list_box_append (GTK_LIST_BOX (dialog->listbox), row);
-		gtk_widget_set_visible (row, TRUE);
 	}
 }
 
