@@ -22,6 +22,7 @@ struct _GsAppVersionHistoryRow
 	GtkWidget	*version_number_label;
 	GtkWidget	*version_date_label;
 	GtkWidget	*version_description_box;
+	GtkWidget	*installed_label;
 };
 
 G_DEFINE_TYPE (GsAppVersionHistoryRow, gs_app_version_history_row, GTK_TYPE_LIST_BOX_ROW)
@@ -83,6 +84,7 @@ gs_app_version_history_row_class_init (GsAppVersionHistoryRowClass *klass)
 	gtk_widget_class_bind_template_child (widget_class, GsAppVersionHistoryRow, version_number_label);
 	gtk_widget_class_bind_template_child (widget_class, GsAppVersionHistoryRow, version_date_label);
 	gtk_widget_class_bind_template_child (widget_class, GsAppVersionHistoryRow, version_description_box);
+	gtk_widget_class_bind_template_child (widget_class, GsAppVersionHistoryRow, installed_label);
 
 	/**
 	 * GsAppVersionHistoryRow:always-expanded:
@@ -115,6 +117,7 @@ gs_app_version_history_row_init (GsAppVersionHistoryRow *row)
  *   or `0` if unknown
  * @version_description: (nullable): Pango Markup for the full human readable
  *   description of the release, or %NULL if unknown
+ * @is_installed: whether the row corresponds to the currently installed version
  *
  * Set information about the release represented by this version history row.
  */
@@ -122,27 +125,24 @@ void
 gs_app_version_history_row_set_info (GsAppVersionHistoryRow *row,
 				     const char *version_number,
 				     guint64     version_date,
-				     const char *version_description)
+				     const char *version_description,
+				     gboolean is_installed)
 {
 	g_autofree char *version_date_string = NULL;
 	g_autofree char *version_date_string_tooltip = NULL;
+	g_autofree char *version_tmp = NULL;
 
 	if (version_number == NULL || *version_number == '\0')
 		return;
 
+	version_tmp = g_strdup_printf (_("Version %s"), version_number);
+	gtk_label_set_label (GTK_LABEL (row->version_number_label), version_tmp);
+
 	if (version_description != NULL && *version_description != '\0') {
-		g_autofree char *version_tmp = NULL;
-		version_tmp = g_strdup_printf (_("New in Version %s"), version_number);
-		gtk_label_set_label (GTK_LABEL (row->version_number_label), version_tmp);
 		gs_description_box_set_text (GS_DESCRIPTION_BOX (row->version_description_box), version_description);
 		gtk_widget_remove_css_class (row->version_description_box, "dim-label");
 	} else {
-		g_autofree char *version_tmp = NULL;
-		const gchar *version_description_fallback;
-		version_tmp = g_strdup_printf (_("Version %s"), version_number);
-		gtk_label_set_label (GTK_LABEL (row->version_number_label), version_tmp);
-		version_description_fallback = _("No details for this release");
-		gs_description_box_set_text (GS_DESCRIPTION_BOX (row->version_description_box), version_description_fallback);
+		gs_description_box_set_text (GS_DESCRIPTION_BOX (row->version_description_box), _("No details for this release"));
 		gtk_widget_add_css_class (row->version_description_box, "dim-label");
 	}
 
@@ -169,6 +169,8 @@ gs_app_version_history_row_set_info (GsAppVersionHistoryRow *row,
 
 	if (version_date_string_tooltip != NULL)
 		gtk_widget_set_tooltip_text (row->version_date_label, version_date_string_tooltip);
+
+	gtk_widget_set_visible (row->installed_label, is_installed);
 }
 
 GtkWidget *
