@@ -34,5 +34,8 @@ void		 gs_repo_row_emit_switch_clicked	(GsRepoRow	*self);
 gboolean	 gs_repo_row_get_related_loaded		(GsRepoRow	*row);
 void		 gs_repo_row_set_related_loaded		(GsRepoRow	*row,
 							 gboolean	 value);
+GCancellable	*gs_repo_row_get_cancellable		(GsRepoRow	*row);
+void		 gs_repo_row_set_cancellable		(GsRepoRow	*row,
+							 GCancellable	*cancellable);
 
 G_END_DECLS
